function [ data, g, data0, u] = ...
                         EP2d_mintime(accuracy, dissType, gridDim, gridSize, tMax, Vpo, npo)
global Ks targ umax umin Dtfactor fx1 fx2 Kavs Naavs
%   [ data, g, data0 ] = FH2d_minE(accuracy, dissType, gridDim, gridSize, tMax)

%
%
% Parameters (all inputs have defaults):
%   accuracy     Controls the order of approximations.
%                  'low'         Use odeCFL1 and upwindFirstFirst (default).
%                  'medium'      Use odeCFL2 and upwindFirstENO2.
%                  'high'        Use odeCFL3 and upwindFirstENO3.
%                  'veryHigh'    Use odeCFL3 and upwindFirstWENO5.
%   dissType     Which type of dissipation to use with the LF approximation?
%                  'global'      Use artificialDissipationGLF (default).
%                  'local'       Use artificialDissipationLLF.
%                  'locallocal'  Use artificialDissipationLLLF.
%   gridDim      Number of dimensions.  Default = 2.
%   gridSize     Number of nodes in each dimension. 
%                  Default = 80 / 2^(gridDim-1).
%                  For HH2D, 151   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   tMax         Final time.  Default = 1.5 / pi^2.
%                             For HH2D, 10.   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%   data         Implicit surface function at tMax.
%   g            Grid structure on which data was computed.
%   data0        Implicit surface function at initial time.

%---------------------------------------------------------------------------
% You will see some executable lines that are commented out.
%   These are included to show some of the options available; modify
%   the commenting to modify the behavior.
  
%---------------------------------------------------------------------------
% Make sure we can see the kernel m-files.
run('C:\Users\dan\Desktop\HJB Toolbox\Examples\addPathToKernel');

%---------------------------------------------------------------------------
% Set defaults.
if(nargin < 1)
  accuracy = 'low';
end

if(nargin < 2)
  dissType = 'global';
end

if(nargin < 3)
  gridDim = 2;
end

if(nargin < 4)
%   gridSize = 80 / 2^(gridDim - 1);
    gridSize = 101;
end

if(nargin < 5)
%   tMax = 1.5 / pi^2;
    tMax = 2000;
end

%---------------------------------------------------------------------------
% Integration parameters.
t0 = 0;                      % Start time.
% alpha = 1.0;                 % Parameter in the Hamiltonian.

% Plotting choices.
plotIntermediate = 1;  % Plot at intermediate timesteps?
Dtfactor = 2000;
tPlot = tMax / Dtfactor;          % Period at which plot should be produced.
singleStep = 0;              % Plot at each timestep (overrides tPlot).

% How close (relative) do we need to get to tMax to be considered finished?
small = 100 * eps;

%---------------------------------------------------------------------------
% What level set should we view?
level = 0;

% Pause after each plot?
pauseAfterPlot = 0;

% Delete previous plot before showing next?
deleteLastPlot = 1;

%---------------------------------------------------------------------------
% Use periodic boundary conditions?
periodic = 0; 


% Create the V-n grid.
% gVn.dim = gridDim;
% gVn.min = [-80;0];
% gVn.N = gridSize;
% if(periodic)
%   gVn.max = (gridSize - 2) / gridSize * g.dim;
%   gVn.bdry = @addGhostPeriodic;
% else
%   gVn.max = [60;1];
%   gVn.bdry = @addGhostExtrapolate;
% end
% gVn = processGrid(gVn);

% Create the grid.
g.dim = gridDim;
% g.min = [(gVn.min(1)-targ(1))/Ks ; gVn.min(2)-targ(2)];
g.min = [5;14]; 
g.N = gridSize;
if(periodic)
  g.max = [19;21];%(gridSize - 2) / gridSize * g.dim;   %19 for full
  g.bdry = @addGhostPeriodic;
else
%   g.max = [(gVn.max(1)-targ(1))/Ks ; gVn.max(2)-targ(2)];
  g.max = [19;20.9];   %19 for full
  g.bdry = @addGhostExtrapolate;
end
g = processGrid(g);


%---------------------------------------------------------------------------
% What kind of display?
switch(g.dim)
 case 1
  displayType = 'plot';
 case 2
  displayType = 'surf';
 case 3
  displayType = 'slice';
 otherwise
  error('Default display type undefined for dimension %d', g.dim);
end

load myaveragecurrentsinterp

for k = 1:40
    NAcurrents(:,:,k) = 1;
    Kcurrents(:,:,1) = 1;
end

% %%%%%%Interpolate Current Data%%%%%%%%
% for k = 1:71
%     for j = 1:70
%         
% 
%         
% bufNa(1,:) = NAcurrents(k,j,:);
% bufK(1,:) = Kcurrents(k,j,:);
% 
% 
% %do NA%%%
% [~,M] = min(abs(bufNa+30));
% buf1 = bufNa(M:length(bufNa));
% p = polyfit(1:length(buf1),buf1,3);
% 
% for m = 1:length(buf1)
%     NAcurrents(k,j,m+M-1) = polyval(p,m);
% end
% %%%%%%
% 
% %         if ((k == 63||k == 64) && j ==64)
% % plot(bufNa)%%%%%%%%%5
% % hold on%%%%%%%%%%%%%%%%%
% % bufNan(1,:) = NAcurrents(k,j,:);%%%%%%%%%5
% % plot(bufNan,'r');%%%%%%%%%%%%%%%%%%%%%%555
% %         end
% 
% %do K
% [~,M] = min(abs(bufK-20));
% buf1K = bufK(M:length(bufK));
% pK = polyfit(1:length(buf1K),buf1K,4);
% 
% for m = 1:length(buf1K)
%     Kcurrents(k,j,m+M-1) = polyval(pK,m);
% end
% 
% %         if ((k == 50||k == 51) && j ==64)
% % plot(bufK)%%%%%%%%%5
% % hold on%%%%%%%%%%%%%%%%%
% % bufNan(1,:) = Kcurrents(k,j,:);%%%%%%%%%5
% % plot(bufNan,'r');%%%%%%%%%%%%%%%%%%%%%%555
% %         end
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%
% clear bufNa bufK buf1 p pK buf1K M
%     end
% end

[Kavs Naavs] = findaveragecurrents(g,Kcurrents,NAcurrents);
RR = size(Kavs);

storagecount = 1;
[fx1 fx2] = EP_f_2d(g);

%---------------------------------------------------------------------------
% Create initial conditions.
%   Extending the 1 and 2 dimensional examples to higher dimension:
%       \phi(x, 0) = 0

% data0Vn = 1*ones(size(gVn.xs{1}));
% tempV = abs(gVn.xs{1} - targ(1));
% tempn = abs(gVn.xs{2} - targ(2));
% r = find(tempV(:,1)==min(min(tempV)));
% c = find(tempn(1,:)==min(min(tempn)));
% data0Vn(r,c) = 0;
% dataVn = data0Vn;

% data0 = 1*ones(size(g.xs{1}));
% data0(r,c) = 0;
% data0(ceil(g.N/2),ceil(g.N/2)) = 0;

Ks = 1;
umax = 10;
umin = -2;
pp = 1;  %tell when to plot

data0 = zeros(size(g.xs{1}));

% Function target set
%load rawtargetsetdata
load targetset15000safe



curve = [Xdata' Ydata'];

data0 = signeddistance(curve,g.xs{1},g.xs{2});

for k = 1:length(data0)
    for j = 1:length(data0)
        data0(k,j) = 1000/(1+exp(-5*(data0(k,j)-1.2)));
    end
end

% pcolor(g.xs{1},g.xs{2},data0)
% hold on
% plot(Xdata,Ydata,'k','linewidth',3)
% colorbar

%plot IC's
surf(g.xs{1},g.xs{2},data0);



data = data0;
Vsol(:,:,storagecount) = data';
storagecount = storagecount + 1;

% for i = 2 : g.dim
%   data = data + g.xs{i};
% end
% data = -cos((pi / g.dim) * data);
% data0 = data;
% pause
% g.xs{1}
% g.xs{2}
% pause
%---------------------------------------------------------------------------
% Set up spatial approximation scheme.
schemeFunc = @termLaxFriedrichs;
schemeData.grid = g;
schemeData.hamFunc = @EP2DHamFunc;
schemeData.partialFunc = @EP2DPartialFunc;
accuracy = 'medium';

% This parameter is not required by termLaxFriedrichs, 
%   but is used by hamFunc and partialFunc.
% schemeData.alpha = alpha;

%---------------------------------------------------------------------------
% Set up time approximation scheme.
integratorOptions = odeCFLset('factorCFL', 0.5, 'stats', 'on');

if(singleStep)
  integratorOptions = odeCFLset(integratorOptions, 'singleStep', 'on');
end

%---------------------------------------------------------------------------
% Choose approximations at appropriate level of accuracy.
switch(accuracy)
 case 'low'
  schemeData.derivFunc = @upwindFirstFirst;
  integratorFunc = @odeCFL1;
 case 'medium'
  schemeData.derivFunc = @upwindFirstENO2;
  integratorFunc = @odeCFL2;
 case 'high'
  schemeData.derivFunc = @upwindFirstENO3;
  integratorFunc = @odeCFL3;
 case 'veryHigh'
  schemeData.derivFunc = @upwindFirstWENO5;
  integratorFunc = @odeCFL3;
 otherwise
  error('Unknown accuracy level %s', accuracy);
end

%---------------------------------------------------------------------------
% What kind of dissipation?
switch(dissType)
 case 'global'
  schemeData.dissFunc = @artificialDissipationGLF;
 case 'local'
  schemeData.dissFunc = @artificialDissipationLLF;
 case 'locallocal'
  schemeData.dissFunc = @artificialDissipationLLLF;
 otherwise
  error('Unknown dissipation function %s', dissFunc);
end
  
%---------------------------------------------------------------------------
% Initialize Display
f = figure;
gforplot = g;
gforplot.xs{1} = Ks*gforplot.xs{1};
gforplot.axis(1:2) = Ks*gforplot.axis(1:2);
% h = visualizeLevelSet(gforplot, data, displayType, level, [ 't = ' num2str(tMax-t0) ]);
h = visualizeLevelSet(gforplot, data, displayType, level);
hold on;
if(gforplot.dim > 1)
   axis(gforplot.axis);
%   daspect([ 1 1 1 ]);
end

% % figure % for generating figure for paper
% % surf(g.xs{1}(50:100,100:170),g.xs{2}(50:100,100:170),data(50:100,100:170))
% shading interp;
% hold on;
% line(Vpo,npo,'Color',[.4 .4 .4],'LineWidth',2); hold on;
% plot(targ(:,1),targ(:,2),'k*','MarkerSize',9,'LineWidth',2);
% set(gca,'FontName','times','FontSize',14,'XColor','k','YColor','k','ZColor','k');
% xlabel('$$V {\rm (mV)}$$','Interpreter','latex','FontSize',14)
% ylabel('$$n$$','Interpreter','latex','FontSize',14)
% zlabel('$$\mathcal{V}(V,n,t)$$','Interpreter','latex','Fontsize',16)
% title(sprintf('$$ t = %1.2f $$',tMax-t0),'Interpreter','latex','Fontsize',18);
% zlim([0,1000]);
JJJ = 1;
% name = sprintf('%i',JJJ);
% dirpath = fullfile('matlab results','value function',name);
% saveas(figure(1), [dirpath,'.fig']) % can put pdf eps etc.
% saveas(figure(1), [dirpath,'.png'])
% % fVn = figure;
% % hVn = visualizeLevelSet(gVn, dataVn, displayType, level, [ 't = ' num2str(t0) ]);
% % hold on;
% % if(gVn.dim > 1)
% %   axis(gVn.axis);
% % %   daspect([ 1 1 1 ]);
% % end

%---------------------------------------------------------------------------
% Loop until tMax (subject to a little roundoff).

j = 1;
tNow = t0;
for i = 1 :1% g.dim
    [ derivL{i}, derivR{i} ] = feval(schemeData.derivFunc, g, data, i);
    derivC{i} = 0.5 * (derivL{i} + derivR{i});
end

%%%%%%%Open and print derivitave data to a file
% fid = fopen(textstring,'wt');
% fprintf(fid,'%f ',derivC{1}(:));
% fprintf(fid,'\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



u{j} = (abs(derivC{1})<=2*Ks*umax).* (-0.5*derivC{1}/Ks) + ...
       (abs(derivC{1})> 2*Ks*umax).* (-sign(derivC{1})*umax);
time(j) = t0;

ufig = figure;
uvis = visualizeLevelSet(g, u{j}, displayType, level, [ 't = ' num2str(t0) ]);

startTime = cputime;
while(tMax - tNow > small * tMax)

  % Reshape data array into column vector for ode solver call.
  y0 = data(:);

  
  % How far to step?
  tSpan = [ tNow, min(tMax, tNow + tPlot) ];
  
  % Take a timestep.
  [ t y ] = feval(integratorFunc, schemeFunc, tSpan, y0,...
                  integratorOptions, schemeData);
  tNow = t(end);

  % Get back the correctly shaped data array
  data = reshape(y, g.shape);
%--------------------------------------------
j = j+1;
for i = 1% g.dim
    [ derivL{i}, derivR{i} ] = feval(schemeData.derivFunc, g, data, i);
    derivC{i} = 0.5 * (derivL{i} + derivR{i});
end

%%print derivitive data to a file
% fprintf(fid,'%f ',derivC{1}(:));
% fprintf(fid,'\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

u{j} = (abs(derivC{1})<=2*Ks*umax).* (-0.5*derivC{1}/Ks) + ...
       (abs(derivC{1})> 2*Ks*umax).* (-sign(derivC{1})*umax);
time(j) = tNow;
%---------------------------------------------

Vsol(:,:,storagecount) = data';
storagecount = storagecount + 1;

if(plotIntermediate)
    if pp == 100
    if(pauseAfterPlot)
      % Wait for last plot to be digested.
      pause;
    end
    
    % Get correct figure, and remember its current view.
    figure(f);
%     figureView = view;

    % Delete last visualization if necessary.
    if(deleteLastPlot)
      delete(h);
    end
    
    %['t = ' num2str(tMax-tNow)]
    % Create new visualization.
    
    h = visualizeLevelSet(gforplot, data, displayType, level );
    shading interp;
    hold on;
    %line(Vpo,npo,'Color',[.4 .4 .4],'LineWidth',2); hold on;
   % plot(targ(:,1),targ(:,2),'k*','MarkerSize',9,'LineWidth',2)
    set(gca,'FontName','times','FontSize',14,'XColor','k','YColor','k','ZColor','k');
    xlabel('$$K {\rm (mV)}$$','Interpreter','latex','FontSize',14)
    ylabel('$$Na$$','Interpreter','latex','FontSize',14)
    zlabel('$$\mathcal{\phi}(K,Na,s)$$','Interpreter','latex','Fontsize',16)
    title(sprintf('$$ t = %1.2f $$',tNow),'Interpreter','latex','Fontsize',18);
    zlim([-2 100])
    %axis([5 15 14 20 -5 20])
    JJJ = JJJ+1;

%     if mod(JJJ,save)==0
%         name = sprintf('%i',JJJ);
%         dirpath = fullfile('matlab results','value function',name);
%         saveas(figure(1), [dirpath,'.fig']) % can put pdf eps etc.
%         saveas(figure(1), [dirpath,'.png'])
%     end
    % Restore view.
%     view(figureView);

    figure(ufig);
    if(deleteLastPlot)
      delete(uvis);
    end

    % Create new visualization.
    uvis = visualizeLevelSet(g, u{j}, displayType, level, ['t = ' num2str(tNow)]);
    pp = 0;
    end
    pp = pp+1;

  end
end

endTime = cputime;
fprintf('Total execution time %g seconds', endTime - startTime);

%---------------------------------------------------------------------------
% Plot the final result, if it has not already appeared.
if(~plotIntermediate)
  % Get correct figure, and remember its current view.
  figure(f);
%   figureView = view;
  
  % Delete last visualization if necessary.
  if(deleteLastPlot)
    delete(h);
  end
  
  % Create new visualization.
%   h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(tNow) ]);
  visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(tNow) ]);
  
  % Restore view.
%   view(figureView);
end
  
save('Vsolution','Vsol','schemeData','g');
fclose(fid);


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function hamValue = EP2DHamFunc(t, data, deriv, schemeData)
global umax umin Ks fx1 fx2 Kavs Naavs
% This function implements the hamFunc prototype to calculate the 
%   analytic Hamiltonian.
% Parameters:
%   t            Time at beginning of timestep (ignored).
%   data         Data array.
%   deriv	 Cell vector of the costate (\grad \phi).
%   schemeData	 A structure (see below).
%
%   hamValue	 The analytic hamiltonian.
%
% schemeData is a structure containing data specific to this Hamiltonian
%   For this function it contains the field(s):
%
%   .grid	 Grid structure.
%
% Ian Mitchell 2/12/04

checkStructureFields(schemeData, 'grid');

% hamValue = schemeData.alpha * ones(size(data));

% [fx1 fx2] = FN_f_2d(schemeData.grid);
 
 
%  khar = 1;
% if abs(deriv{1})<=2*Ks*umax
%     hamValue = -deriv{1}.*fx1 - deriv{2}.*fx2 + 1/4*(deriv{1}.*1/Ks).^2  ;
% else
%     hamValue = -deriv{1}.*fx1 - deriv{2}.*fx2 - umax^2 + abs(deriv{1}*1/Ks)*umax;
% end
% hamValue = deriv{1}.*fx1 + deriv{2}.*fx2 - 1./4.*(deriv{1}.*1/Ks).^2;
RR = size(Kavs);

buf1 = zeros(length(deriv{1}),length(deriv{1}),RR(3));
buf2 = buf1;
for k = 1:RR(3)
    W = 20/(RR(3)-1);
    if -10+W*(k -1) <=umax+.001   &&  -10+W*(k -1) >=umin -.001
    buf1(:,:,k) = deriv{1};
    buf2(:,:,k) = deriv{2};
    
    
    energybuf(:,:,k) = ones(size(deriv{1}))*(.001*(-10+W*(k -1))^2);
    
    else
    buf1(:,:,k) = Kavs(:,:,k)*1000;
    buf2(:,:,k) = Naavs(:,:,k)*1000;
    
    energybuf(:,:,k) = ones(size(deriv{1}))*(100^2);
    end
end

%with External Control%
hamValue = (-deriv{1}.*fx1 - deriv{2}.*fx2 - min(energybuf + buf1.*Kavs + buf2.*Naavs,[],3));




%%%%%without any external control%%%%%%%%%%
% newbuf = deriv{1}.*Kavs(:,:,51) + deriv{2}.*Naavs(:,:,51);
% hamValue = (-deriv{1}.*fx1 - deriv{2}.*fx2 - newbuf);

%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function alpha = ...
               EP2DPartialFunc(t, data, derivMin, derivMax, schemeData, dim)
global umax Ks fx1 fx2 Kavs Naavs


% This function implements the partialFunc prototype to calculate the
%   extrema of the absolute value of the partials of the analytic
%   Hamiltonian with respect to the costate (gradient).
%
% 
% Parameters:
%   t            Time at beginning of timestep (ignored).
%   data         Data array.
%   derivMin	 Cell vector of minimum values of the costate (\grad \phi).
%   derivMax	 Cell vector of maximum values of the costate (\grad \phi).
%   schemeData	 A structure (see below).
%   dim          Dimension in which the partial derivatives is taken.
%
%   alpha	 Maximum absolute value of the partial of the Hamiltonian
%		   with respect to the costate in dimension dim for the 
%                  specified range of costate values.
%		   Note that alpha can (and should) be evaluated separately
%		   at each node of the grid.
%
% schemeData is a structure containing data specific to this Hamiltonian
%   For this function it contains the field(s):
%
%   .grid	 Grid structure.
%
% Ian Mitchell 2/12/04

checkStructureFields(schemeData, 'grid');

%[fx1 fx2] = FN_f_2d(schemeData.grid);
Vx1 = max(abs(derivMin{1}), abs(derivMax{1}));
Vx2 = max(abs(derivMin{2}), abs(derivMax{2}));
switch dim
  case 1
      alpha = abs(fx1) + max(abs(Kavs),[],3);
  case 2
      alpha = abs(fx2) + max(abs(Naavs),[],3);
end
      

